<?
/**
* This gateway was developed by and is copyright of Axsys Technology Group.
* Sections of code may be copyrighted to other parties (eg. where sample code was used
* from the Quantum documentation). All parts (of this gateway only)
* written by Axsys Technology Group are licensed for use in any way you wish, as long as this
* copyright message remains intact, and without any guarantee of any sort - use at your own risk.
*
* Description:
* This file is called to process Merchant account functionality. It is called from the Order form,
* and provides the card capture fields. It also interacts with the Quantum gateway provided forms
* and returns results to your provided URLs.
*
*	Keep in mind, that the following fields are defined and populated by nBill before it calls 
* this gateway script:
*
* $g_tx_id -	Gateway ID for the transaction (this will be needed if the payment gateway posts 
* 						back to the merchant's website, to enable nBill to identify what the payment is for.
*
* $currency -	The 3 character code for the currency of the transaction (eg. USD, or GBP).
*
* $total_gross - The full amount that needs to be paid (including any setup fees).
*
* $regular_total_gross - The full amount that needs to be paid on a recurring basis 
*												 after the initial payment (this will only be different from 
*												 $total_gross if there is a setup fee defined for an ordered 
*												 product, or a discount is being applied to the first payment only).
*
* $orders - An array of items that have been ordered:
*    $orders[<index>]['product_code'] - The SKU of the item.
*    $orders[<index>]['product_name'] - The name of the item.
*    $orders[<index>]['net_price'] - The net price of the item.
*    $orders[<index>]['quantity'] - The number of units ordered.
*
* $payment_frequency - A 2-character code indicating how often to make repeat payments.  
* 	 Possible values are:
* 			AA - One-off
* 			BB - Weekly
* 			CC - Monthly
* 			DD - Quarterly
* 			EE - Annually
* 			FF - Bi-annually
* 			GG - Five-yearly
* 			HH - Ten-yearly
*
*	TODO: Add the ability to allow vendors to use a test account and switch between the two.
* 			This will require that I provide more fields to capture gwlogin and perhaps
*				the restriction key, as well as a Y/N field to determine if they want LIVE or
*				TEST account.
*/
/** Ensure this file is being included by a parent file */
defined( '_VALID_MOS' ) or die( 'Direct Access to this location is not allowed.' );

// Define the language variable globally, so that we can access it from anywhere.
// This will be filled outside of this module.
global $mosConfig_lang, $mosConfig_live_site, $mosConfig_sitename;
if (file_exists(INV_ADMIN_BASE_PATH . "/admin.gateway/admin.quantum/quantum_" . $mosConfig_lang . ".php"))
{
	include_once(INV_ADMIN_BASE_PATH . "/admin.gateway/admin.quantum/quantum_" . $mosConfig_lang . ".php");
}
else 
{
	if (file_exists(INV_ADMIN_BASE_PATH . "/admin.gateway/admin.quantum/quantum_english.php"))
	{
		include_once(INV_ADMIN_BASE_PATH . "/admin.gateway/admin.quantum/quantum_english.php");
	}
}

//Define support functions, if necessary
if (!function_exists("hmac")) // Quantum can be set to use hash-value and md5.
{
	function hmac($key, $data)
	{
		 // RFC 2104 HMAC implementation for php.
		 // Creates an md5 HMAC.
		 // Eliminates the need to install mhash to compute a HMAC
		 // Hacked by Lance Rushing
		 $b = 64; // byte length for md5
		 if (strlen($key) > $b) {
				 $key = pack("H*",md5($key));
		 }
		 $key  = str_pad($key, $b, chr(0x00));
		 $ipad = str_pad('', $b, chr(0x36));
		 $opad = str_pad('', $b, chr(0x5c));
		 $k_ipad = $key ^ $ipad ;
		 $k_opad = $key ^ $opad;
		 return md5($k_opad  . pack("H*", md5($k_ipad . $data)));
	}
}

// Let's retrueve the QUANTUM specific values specified in the gateway setup.
$sql = "SELECT * FROM #__inv_payment_gateway WHERE gateway_id = 'quantum'";
$database->setQuery($sql);
$quantum_fields = $database->loadAssocList('g_key');
if (!array_key_exists('gwlogin', $quantum_fields))
{
	//loadAssocList has not worked - I don't know why, but this happens on some servers (so we need to rebuild the array with the correct keys)
	$quantum_fields = array();
	$alt_quantum_fields = $database->loadObjectList();
	foreach ($alt_quantum_fields as $alt_quantum_field)
	{
		$quantum_fields[$alt_quantum_field->g_key] = array();
		$quantum_fields[$alt_quantum_field->g_key]['g_key'] = $alt_quantum_field->g_key;
		$quantum_fields[$alt_quantum_field->g_key]['g_value'] = $alt_quantum_field->g_value;
	}
}

// We'll need to create this array for storing the params that we'll be posting
// to the gateway processing form.
$params = array();

// Determine the payment frequency we'll be using in this case.
// The order form will need to provide a selection field or a hidden field that contains
// the recipe ID, which should be named RID.
$is_one_off = $payment_frequency == "AA" ? true : false;

if($is_one_off == false) {
	$params['RID']               = mosGetParam($_POST, 'RID');
	$params['override_recur']    = $regular_total_gross;
	$params['initial_amount']    = $total_gross;
	$params['OverRideRecureDay'] = $quantum_fields['override_recurring_day']['g_value'];
}

// Mandatory Fields
$params['gwlogin'] = $quantum_fields['gwlogin']['g_value']; // Gateway login ID.
$params['RestrictKey'] = $quantum_fields['restriction_key']['g_value']; // Restriction Key (if provided).

// Specify where to return
$params['post_return_url_approved'] = $mosConfig_live_site . "/" . $quantum_fields['return_url']['g_value'];
$params['post_return_url_declined'] = $mosConfig_live_site . "/" . $quantum_fields['return_url']['g_value'];
// $params['ResponseMethod'] = "GET";

// Store the total gross (unless we're performing a trial only (which would be charged 
// on follow-up months if it is a recurring charge).
$params['amount'] = $total_gross;
if ($total_gross == 0)
{
	$params['trans_type'] = "auth_only"; 			// Free trial, so just authorize the card
	$params['amount'] = $regular_total_gross; // On-going amount (may different from total gross).
}
else 
{
	$params['trans_type'] = "auth_sale";			// Charge the whole thing now and perform AVS check.
}

/** 
 * Optional Fields
 */

// Should we override the email customer flag in the Quantum Gateway account?
if (trim($quantum_fields['override_email_customer']['g_value']) != "")
{
	$params['override_email_customer'] = $quantum_fields['override_email_customer']['g_value'];
}

// Should we override the trans email flag in the Quantum Gateway account?
if (trim($quantum_fields['override_trans_email']['g_value']) != "")
{
	$params['override_trans_email'] = $quantum_fields['override_trans_email']['g_value'];
}

// If you wish to display a logo, let's tell Quantum where it can fetch it from.
if (strlen($quantum_fields['company_logo']['g_value']) != 8 && substr($quantum_fields['company_logo']['g_value'], 0, 8) == "https://")
{
	$params['company_logo'] = $quantum_fields['company_logo']['g_value'];
}

// If you want to change the background color of the Quantum form, let's pass on the color.
if (trim($quantum_fields['bg_color']['g_value']) != "")
{
	$params['bg_color'] = $quantum_fields['bg_color']['g_value'];
}

// If you want to change the text color of the Quantum form, let's pass on the color.
if (trim($quantum_fields['txt_color']['g_value']) != "")
{
	$params['txt_color'] = $quantum_fields['txt_color']['g_value'];
}

// Anything sent in this form field will be displayed at the top of the receipt page,
// right before the order summary.
if (trim($quantum_fields['header_receipt']['g_value']) != "")
{
	$params['header_receipt'] = $quantum_fields['header_receipt']['g_value'];
}

// Anything sent in this form field will be shown at the top of the checkout/payment page.
if (trim($quantum_fields['payment_heading']['g_value']) != "")
{
	$params['payment_heading'] = $quantum_fields['payment_heading']['g_value'];
}

// Specify the URL that we need to post to, in order to reach the Gateway.
$quantum_url = "https://" . $quantum_fields['quantum_url']['g_value'];

/**
 * Grab the customer's values we'll need.
 */
$params['ID'] = $g_tx_id;
$params['cust_id'] = mosGetParam($_POST, 'INV_CORE_username');
$params['FNAME'] = mosGetParam($_POST, 'INV_CORE_first_name');
$params['LNAME'] = mosGetParam($_POST, 'INV_CORE_last_name');
$params['company'] = mosGetParam($_POST, 'INV_CORE_company_name');
$params['BADDR1'] = mosGetParam($_POST, 'INV_CORE_address_1');
$params['BCITY'] = mosGetParam($_POST, 'INV_CORE_town');
$params['BSTATE'] = mosGetParam($_POST, 'INV_CORE_state');
$params['BZIP1'] = mosGetParam($_POST, 'INV_CORE_postcode');
$params['BCOUNTRY'] = mosGetParam($_POST, 'INV_CORE_country');
$params['BCUST_EMAIL'] = mosGetParam($_POST, 'INV_CORE_email_address');
$params['trans_method'] = "CC"; //TODO: Make this selectable (to offer EFT Payments.
$params['customer_ip'] = nbGetUserIP();	// This could be useful for catching fraud info.

// Collect Description Information.
foreach ($orders as $order)
{
	if (isset($params['description']))
	{
		$params['description'] .= "; " . $order['product_name'];
	}
	else 
	{
		$params['description'] = $order['product_name'];
	}
}

// First check if we are dealing with a one-time payment and if so, do we have an HTTPS connection.
// If One-Time Payment and NOT HTTPS, then go to the Web Order form.
if($_SERVER['SERVER_PORT'] != 443) // Redirect to Quantum's Web Order form.
{
	//If not using SSL it is not safe (or if SSL posting is not supported, it is not possible) to 
	//echo the results, so user will have to be asked to POST to the PSP's secure server (we cannot 
	//reliably redirect the user transparently - even if we attempt this, their browser is supposed 
	//to ask the user for confirmation (some do, some don't, some won't even do the redirect). As 
	//this confirmation request is generated by the browser, most users will just be confused by it. 
	//To work around this, we will output a form but immediately submit it via javascript so that
	//the user still does not have to do anything. If javascript is disabled, the form will be
	//displayed - so to make it obvious to the user what they are expected to do, we will ask them 
	//to click a button to confirm - the button submits the form directly to the PSP.
	?>
	<script type="text/javascript">
	<!--
		document.write("<div style=\"display:none;\">");
	-->
	</script>
	<?php //Show previous button (if summary already being used, skip past it) ?>
	<div style="float:left">
		<form action="index.php" method="post" name="order_form" id="order_form">
		  <input type="hidden" name="option" value="com_netinvoice" />
			<?php 
			echo "<input type=\"hidden\" name=\"action\" value=\"" . mosGetParam($_POST, 'action') . "\" />";
			echo "<input type=\"hidden\" name=\"task\" value=\"" . mosGetParam($_POST, 'task') . "\" />";
			echo "<input type=\"hidden\" name=\"cid\" value=\"" . mosGetParam($_POST, 'cid') . "\" />";
			echo "<input type=\"hidden\" name=\"summary_screen\" value=\"1\" />";
			echo "<input type=\"hidden\" name=\"summary_posted_values\" value=\"" . base64_encode(serialize($_POST)) . "\" />";
			echo "<input type=\"hidden\" name=\"back\" value=\"1\" />";
			?>
			<input type="submit" name="previous" value="<?php echo INV_PREV; ?>" />
		</form>
	</div>
	<div style="float:right">
		<form action="<?php echo $quantum_url; ?>" method="POST" name="summary_form" id="summary_form">
			<?php
				foreach ($params as $key=>$value)
				{
					echo "<input type=\"hidden\" name=\"$key\" value=\"$value\" />";
				}
			?>
			<input type="submit" name="form_submit" value="<?php echo INV_QUANTUM_SUBMIT; ?>" />
		</form>
	</div>
	<script type="text/javascript">
	<!--
		document.write("</div>");
		document.getElementById('summary_form').submit();
	-->
	</script>
	<?php
}
// Regardless of whether One-time payment or Recurring, as long as we have an HTTPS
// connection, we're ready to display our own form.
else 
{
	include_once(INV_ADMIN_BASE_PATH . "/admin.gateway/admin.quantum/quantum_card_capture.php");
	show_card_details_screen($quantum_url, $params, $billing_name, "", $quantum_fields['payment_heading']['g_value'], ""/*$quantum_fields['payment_footer']['g_value']*/);
}
$abort = true; // Don't display the thank you message.
?>
