<?
/**
* This gateway was developed by and is copyright of Axsys Technology Group.
* Sections of code may be copyrighted to other parties (eg. where sample code was used
* from the Quantum documentation). All parts (of this gateway only)
* written by Axsys Technology Group are licensed for use in any way you wish, as long as this
* copyright message remains intact, and without any guarantee of any sort - use at your own risk.
*
* This file provides the callback for the Payment Gateway. This is the file that 
* will be processed when the payment service provider does a callback. The callback 
* URL will typically need to look like the following: 
* 'http://www.<domain name here>.com/index.php?option=com_netinvoice&action=gateway&gateway=[gateway]'.
*/

/** Ensure this file is being included by a parent file */
defined( '_VALID_MOS' ) or die( 'Direct Access to this location is not allowed.' );

// Define the language variable globally, so that we can access it from anywhere.
// This will be filled outside of this module.
global $mosConfig_lang, $mosConfig_live_site, $mosConfig_sitename;
if (file_exists(INV_ADMIN_BASE_PATH . "/admin.gateway/admin.quantum/quantum_" . $mosConfig_lang . ".php"))
{
	include_once(INV_ADMIN_BASE_PATH . "/admin.gateway/admin.quantum/quantum_" . $mosConfig_lang . ".php");
}
else 
{
	if (file_exists(INV_ADMIN_BASE_PATH . "/admin.gateway/admin.quantum/quantum_english.php"))
	{
		include_once(INV_ADMIN_BASE_PATH . "/admin.gateway/admin.quantum/quantum_english.php");
	}
}

//Define support functions, if necessary
if (!function_exists("hmac")) // Quantum can be set to use hash-value and md5.
{
	function hmac($key, $data)
	{
		 // RFC 2104 HMAC implementation for php.
		 // Creates an md5 HMAC.
		 // Eliminates the need to install mhash to compute a HMAC
		 // Hacked by Lance Rushing
		 $b = 64; // byte length for md5
		 if (strlen($key) > $b) {
				 $key = pack("H*",md5($key));
		 }
		 $key  = str_pad($key, $b, chr(0x00));
		 $ipad = str_pad('', $b, chr(0x36));
		 $opad = str_pad('', $b, chr(0x5c));
		 $k_ipad = $key ^ $ipad ;
		 $k_opad = $key ^ $opad;
		 return md5($k_opad  . pack("H*", md5($k_ipad . $data)));
	}
}

// Define the method if it hasn't been defined already (depends on whether this is a callback or not)
if (!function_exists("get_occurrences"))
{
	function get_occurrences($start_date, $end_date, $payment_frequency)
	{
		//Return the number of payments that will be made between start and end dates.
		$occurrences = 0;
		$next_date = $start_date;
		while ($next_date < $end_date)
		{
			$occurrences++;
			$next_date = get_next_payment_date($start_date, $next_date, $payment_frequency);
		}
		return $occurrences;
	}
}

// Retrieve the g_tx_id.
$g_tx_id = intval(mosGetParam($_REQUEST, 'ID')); // g_tx_id is stored in ID.

// Let's retrieve the QUANTUM specific values specified in the gateway setup.
$sql = "SELECT * FROM #__inv_payment_gateway WHERE gateway_id = 'quantum'";
$database->setQuery($sql);
$quantum_fields = $database->loadAssocList('g_key');
if (!array_key_exists('gwlogin', $quantum_fields))
{
	//loadAssocList has not worked - I don't know why, but this happens on some servers (so we need to rebuild the array with the correct keys)
	$quantum_fields = array();
	$alt_quantum_fields = $database->loadObjectList();
	foreach ($alt_quantum_fields as $alt_quantum_field)
	{
		$quantum_fields[$alt_quantum_field->g_key] = array();
		$quantum_fields[$alt_quantum_field->g_key]['g_key'] = $alt_quantum_field->g_key;
		$quantum_fields[$alt_quantum_field->g_key]['g_value'] = $alt_quantum_field->g_value;
	}
}

// Quantum does not return the currency or amount 
// We'll need to look it up from our own records, or default to USD.
$sql = "SELECT currency, total_gross FROM #__inv_pending_orders 
				INNER JOIN #__inv_gateway_tx 
				ON #__inv_pending_orders.id = #__inv_gateway_tx.pending_order_id 
				WHERE #__inv_gateway_tx.id = $g_tx_id";
$database->setQuery($sql);
$order_fields = NULL;
$database->loadObject($order_fields);

$amount = $order_fields->total_gross;
$currency = $order_fields->currency;
if (strlen(trim($currency)) == 0)
{
	$currency = "USD";
}
		
// echo "Amount = " . $amount . "<br>";
// echo "Currency = " . $currency . "<br>";

/**
 * Process the result from the PSP.
 */
$recurring = false;
$error_message = "";
$warning_message = "";
$auth_failure = false;

// Uncomment the next 4 lines to view the results from the PSP.
// echo "************ Request ***************";
// print_r ($_REQUEST);
// echo "************ Post ***************";
// print_r ($_POST);

// Check we have an authentic callback.
$gwlogin = $quantum_fields['gwlogin']['g_value'];
$transID = mosGetParam($_REQUEST, 'transID');

if(!$amount)
{
	$amount = "0.00";
}

$hash_value = $quantum_fields['md5hash_value']['g_value'];
$hash_value .= $gwlogin . $transID . $amount; // Our composed value.
$psp_hash = mosGetParam($_REQUEST, "md5_hash"); // Value from PSP.

// Did we have a match or not?
if (strtoupper(md5($hash_value)) != strtoupper($psp_hash))
{
	$error_message = INV_QUANTUM_HASH_MISMATCH . " [unhashed: $hash_value; calculated: " . strtoupper(md5($hash_value)) . "; hash: " . strtoupper($psp_hash) . "]";
}
else 
{
	// If we got here, we had an md5_hash match...let's proceed.
	$trans_result = mosGetParam($_REQUEST, "trans_result");
	
	if ($trans_result != "APPROVED")
	{
		//Not authorized.
		$auth_failure = true;
		$error_message = mosGetParam($_REQUEST, 'decline_reason');
		if(strlen($error_message) == 0)
			$error_message = "<p><strong>Transaction was declined.</strong></p>";
	}
	
	if ((strlen($error_message) == 0) && ($auth_failure != true))
	{
		//Do the nBill processing
		gateway_processing($g_tx_id, $amount, $currency, $warning_message, $error_message, "", "Txn id: $g_tx_id");
	}
}

$redirect_url = "";
if (strlen($error_message) > 0)
{
	if ($auth_failure)
	{
		// TODO: Provide a failure_url.
		if (isset($quantum_fields['failure_url']['g_value']) && strlen($quantum_fields['failure_url']['g_value']) > 0)
		{
			if (substr($quantum_fields['failure_url']['g_value'], 0, 4) == "http")
			{
				$redirect_url = $quantum_fields['failure_url']['g_value'];
			}
			else 
			{
				$redirect_url = $mosConfig_live_site . "/" . $quantum_fields['failure_url']['g_value'];
			}
		}
	}
	else 
	{
		if (isset($quantum_fields['error_url']['g_value']) && strlen($quantum_fields['error_url']['g_value']) > 0)
		{
			if (substr($quantum_fields['error_url']['g_value'], 0, 4) == "http")
			{
				$redirect_url = $quantum_fields['error_url']['g_value'];
			}
			else 
			{
				$redirect_url = $mosConfig_live_site . "/" . $quantum_fields['error_url']['g_value'];
			}
		}
	}
}
else 
{
	if (isset($quantum_fields['success_url']['g_value']) && strlen($quantum_fields['success_url']['g_value']) > 0)
	{
		if (substr($quantum_fields['success_url']['g_value'], 0, 4) == "http")
		{
			$redirect_url = $quantum_fields['success_url']['g_value'];
		}
		else 
		{
			$redirect_url = $mosConfig_live_site . "/" . $quantum_fields['success_url']['g_value'];
		}
	}
}
finish_gateway_processing("", $error_message, @$quantum_fields['add_debug_info']['g_value'], $redirect_url, $g_tx_id);

?>
